@@ -7,6 +7,7 @@ import os
 import warnings
 from abc import abstractmethod
 from collections import abc
+from enum import Enum
 from functools import wraps
 from typing import cast, ClassVar, Mapping, Union, Optional, Any, Callable, \
     Tuple, Protocol
@@ -16,14 +17,15 @@ import requests
 import attr
 from attr import attrs, fields_dict, NOTHING
 from ibm_cloud_sdk_core import BaseService, DetailedResponse
-from ibm_cloud_sdk_core.authenticators import Authenticator, IAMAuthenticator, CloudPakForDataAuthenticator
+from ibm_cloud_sdk_core.authenticators import Authenticator, IAMAuthenticator, \
+    CloudPakForDataAuthenticator, BearerTokenAuthenticator
 import ibm_boto3
 from ibm_botocore.client import Config
 
 from .client_errors import JsonParsingError, MissingValueError, \
     OfCpdPathError, NoWmlInstanceError, WmlServiceNameUnknownTypeError, \
     WmlServiceNameNoPrefixError, WmlServiceCNameNotValidError, \
-    PublicCloudOnlyError, FilesResultsNotSupportedError
+    PublicCloudOnlyError, FilesResultsNotSupportedError, NoSuchOverloadError
 from .cpd_paths import CpdScope
 from .utils import validate_type, \
     get_storage_config_field, get_scope_response_field, get_credentials_field
@@ -37,6 +39,11 @@ def public_cloud_only(fn: Callable) -> Callable:
         return fn(self, *args, **kwargs)
     return inner
 
+class AuthMethod(Enum):
+    APIKEY = 'apikey'
+    BEARER_TOKEN = 'bearer_token'
+    ANY = 'any'
+
 class WSPipelines(BaseService):
     """Watson Studio Pipelines client
 
@@ -57,6 +64,7 @@ class WSPipelines(BaseService):
 
     SDK_NAME = 'ibm-watson-studio-pipelines'
 
+
     @classmethod
     def new_instance(cls, *, service_name: str = None, url: str = None) -> 'WSPipelines':
         """
@@ -69,27 +77,44 @@ class WSPipelines(BaseService):
         """
         Return a new Watson Studio Pipelines client for the specified API key.
         """
-        if apikey is None:
-            apikey = os.environ.get('APIKEY', None)
-        return cls(apikey=apikey, service_name=service_name, url=url)
+        return cls(
+            apikey=apikey,
+            auth_method=AuthMethod.APIKEY,
+            service_name=service_name,
+            url=url,
+        )
+
+    @classmethod
+    def from_token(cls, bearer_token: str = None, *, service_name: str = None, url: str = None) -> 'WSPipelines':
+        """
+        Return a new Watson Studio Pipelines client for the specified bearer token.
+        """
+        return cls(
+            bearer_token=bearer_token,
+            auth_method=AuthMethod.BEARER_TOKEN,
+            service_name=service_name,
+            url=url,
+        )
 
     def __init__(
         self,
         apikey: str = None,
         *,
+        bearer_token: str = None,
         service_name: str = None,
         url: str = None,
+        auth_method: AuthMethod = None,
     ):
+
         url = self._get_cpd_api_url(url)
         validate_type(url, "url", str)
 
-        if apikey is None:
-            apikey = os.environ.get('APIKEY', None)
-            if apikey is None:
-                raise MissingValueError('APIKEY')
-        validate_type(apikey, "apikey", str)
-
-        authenticator, is_public = self._get_authenticator(apikey, url)
+        authenticator, is_public = self._get_authenticator(
+            auth_method,
+            apikey=apikey,
+            bearer_token=bearer_token,
+            url=url,
+        )
 
         if service_name is None:
             service_name = self.DEFAULT_SERVICE_NAME
@@ -104,6 +129,68 @@ class WSPipelines(BaseService):
         self.configure_service(service_name)
         self.is_public = is_public
 
+    def _get_authenticator(
+            self,
+            auth_method: Optional[AuthMethod],
+            apikey: Optional[str],
+            bearer_token: Optional[str],
+            url: str,
+    ) -> Tuple[Authenticator, bool]:
+        def censor_value(value: Optional[Any]) -> Optional[str]:
+            if value is None:
+                return None
+            return '...'
+
+        def no_such_overload() -> NoSuchOverloadError:
+            class_name = type(self).__name__
+            kwargs = {
+                'apikey': censor_value(apikey),
+                'bearer_token': censor_value(bearer_token),
+                'auth_method': auth_method,
+            }
+            kwargs = {k: v for k, v in kwargs.items() if v is not None}
+            return NoSuchOverloadError(class_name, [], kwargs)
+
+        if apikey is not None and bearer_token is not None:
+            raise no_such_overload()
+
+        if auth_method == AuthMethod.APIKEY:
+            if bearer_token is not None:
+                raise no_such_overload()
+            if apikey is None:
+                apikey = os.environ.get('APIKEY', None)
+            if apikey is None:
+                raise MissingValueError('APIKEY')
+            return self._get_authenticator_from_api_key(apikey, url)
+
+        elif auth_method == AuthMethod.BEARER_TOKEN:
+            if apikey is not None:
+                raise no_such_overload()
+            if bearer_token is None:
+                bearer_token = os.environ.get('USER_ACCESS_TOKEN')
+            if bearer_token is None:
+                raise MissingValueError('USER_ACCESS_TOKEN')
+            return self._get_authenticator_from_bearer_token(bearer_token, url)
+
+        elif auth_method == AuthMethod.ANY or auth_method is None:
+            if apikey is not None:
+                return self._get_authenticator_from_api_key(apikey, url)
+            elif bearer_token is not None:
+                return self._get_authenticator_from_bearer_token(bearer_token, url)
+
+            apikey = os.environ.get('APIKEY', None)
+            if apikey is not None:
+                return self._get_authenticator_from_api_key(apikey, url)
+
+            bearer_token = os.environ.get('USER_ACCESS_TOKEN')
+            if bearer_token is not None:
+                return self._get_authenticator_from_bearer_token(bearer_token, url)
+
+            # should provide either APIKEY or USER_ACCESS_TOKEN, usually APIKEY
+            raise MissingValueError('APIKEY')
+        else:
+            raise no_such_overload()
+
     def _get_iam_url(self, url: str) -> Tuple[str, bool]:
         validate_type(url, "url", str)
         if not url.startswith("https://"):
@@ -122,7 +209,7 @@ class WSPipelines(BaseService):
             iam_url = url + "/icp4d-api/v1/authorize"
         return iam_url, is_public
 
-    def _get_authenticator(self, apikey: str, url: str) -> Tuple[Authenticator, bool]:
+    def _get_authenticator_from_api_key(self, apikey: str, url: str) -> Tuple[Authenticator, bool]:
         validate_type(apikey, "api_key", str)
         iam_url, is_public = self._get_iam_url(url)
         if is_public:
@@ -131,6 +218,12 @@ class WSPipelines(BaseService):
             auth = CloudPakForDataAuthenticator("admin", url=iam_url, apikey=apikey, disable_ssl_verification=True)
         return auth, is_public
 
+    def _get_authenticator_from_bearer_token(self, bearer_token: str, url: str) -> Tuple[Authenticator, bool]:
+        validate_type(bearer_token, "bearer_token", str)
+        iam_url, is_public = self._get_iam_url(url)
+        auth = BearerTokenAuthenticator(bearer_token=bearer_token)
+        return auth, is_public
+
     @classmethod
     def get_output_artifact_map(cls) -> Mapping[str, str]:
         """Get output artifacts key-value map from env-var OF_OUTPUT_ARTIFACTS."""
