@@ -2,7 +2,7 @@
 
 import logging
 from typing import Union, Iterable, Any, Sequence, TYPE_CHECKING, Optional, \
-    overload
+    overload, Mapping
 
 if TYPE_CHECKING:
     from .cpd_paths import CpdScope
@@ -34,6 +34,18 @@ class PublicCloudOnlyError(ClientError):
         msg = f'{feature_str} is only available on public cloud, but the URL provided is private cloud.'
         super().__init__(msg, None)
 
+class NoSuchOverloadError(ClientError):
+    def __init__(self, fn: str, args: Sequence[Any], kwargs: Mapping[str, Any], reason: Union[str, Exception] = None):
+        args_str = ', '.join([str(arg) for arg in args])
+        kwargs_str = ', '.join([f'{k}={v}' for k,v in kwargs.items()])
+        if args_str != '' and kwargs_str != '':
+            args_kwargs_str = args_str + ', ' + kwargs_str
+        else:
+            args_kwargs_str = args_str + kwargs_str
+
+        msg = f"No overload: {fn}({args_kwargs_str})"
+        super().__init__(msg, reason)
+
 class MissingValueError(ClientError):
     def __init__(self, value_name: Union[str, Iterable[str]], reason: Union[str, Exception] = None):
         if isinstance(value_name, str):
