@@ -142,7 +142,11 @@ class CpdPath:
         if path.count("?") > 0:
             raise TypeError("Question mark can only be used once.")
 
-        result = CpdPath(scope = scope, path = path, query = query)
+        # what kind of path is it?
+        if len(parts) > 3 and parts[0] == 'connections' and parts[2] == 'files':
+            result = CpdPathToConnectionFile(scope=scope, path=path, query=query)
+        else:
+            result = CpdPath(scope=scope, path=path, query=query)
         return result
 
     def resource_id(self) -> Optional[str]:
@@ -160,3 +164,16 @@ class CpdPath:
         if self.scope is not None:
             return evolve(self)
         return evolve(self, scope=cpd_scope)
+
+@attrs(auto_attribs=True, frozen=True, kw_only=True)
+class CpdPathToConnectionFile(CpdPath):
+    """CPD path pointing at a file inside some connection"""
+
+    def resource_id(self) -> Optional[str]:
+        return self.path.split('/')[1]
+
+    def bucket_name(self) -> str:
+        return self.path.split('/')[3]
+
+    def file_path(self) -> str:
+        return '/' + '/'.join(self.path.split('/')[4:])
\ No newline at end of file
